-- --------------------------=( Script Information )=--------------------------
-- Name :         Callbacks Logger
-- Version :      0.10
-- Last Changed : 2009/Mar/05
-- Built for :    3ds max r5+
--
-- --------------------------------=( Summary )=-------------------------------
-- The callbacks logger provides an easy way to log all the common callback
-- events, managed from a small user interface.
-- Use the callbacks logger to see if activities cause any callbacks to be
-- triggered and if so, which one(s).
--
-- ---------------------------=( Copyright Notice )=---------------------------
-- by Richard Annema <rannema@splutterfish.com>
--
-- ---------------------------=( Acknowledgements )=---------------------------
-- --
--
-- -----------------------------=( Installation )=-----------------------------
-- Simply run this file.  The Callbacks Logger can then be found in the
-- SplutterFish macroscripts category.
-- For permanent accessibility, place this file in the Startup scripts folder.
--
-- -----------------------------=( Documentation )=----------------------------
-- UI:
--   log events - button
---    toggle this button to enable/disable logging
--   listener | window - radiobuttons
--     set whether events should be logged to the Listener or to a separate
--     window (a new window opens each time the logger is activated)
--   events - multilistbox
--     select the events to log here.  Defaults to all events.
--   expression for changeHandlers - edittext
--     an expression that shoudl evaluate to a MaxWrapper (collection) to use
--     for the 'change handler' events.  Defaults to the currently selected
--     objects.
--
--   Changing any of the UI options will automatically re-set the montor.
-- ----------------------------------------------------------------------------
-- Scripting:
--   <OK>cbLogger.start types:<array> chObj: into:<stream>
--     starts the logging process
--   Parameters:
--     types
--       An array of event types.  Valid types: #all, #nodeEvent, #animation,
--       #renderer, #imageViewer, #rendering, #fileLink, #file, #xrefs, #layer,
--       #manipulate, #modpanel, #modifier, #mtllib, #material, #node, #system,
--       #radiosity, #undo, #schematicView, #assetBrowser, #namedSelSet,
--       #direct3d, #changeHandlers, #redrawViews, #time
--       If #all is included, all events will be logged.
--       Default: #(#all)
--     chObj
--       A MaxWrapper or Array of MaxWrappers to associate the change handler
--       events with.  Required if types includes #changeHandlers (or #all).
--       Default: current object selection
--     into
--       Any valid and open stream object (windowStream, stringStream,
--       fileStream).
--       Default: the Listener (windowStream)
--   Returns:
--     OK
--
--   <OK>cbLogger.stop()
--     stops the logging process
--   Returns:
--     OK
-- ----------------------------------------------------------------------------
-- Output:
--   The output begins each event line with the event category:
--     GE = General Event callback
--     NE = Node Event callback
--     CH = Change Handler
--     RV = RedrawViews callback
--     TM = Time (change) callback
--
--   This is followed by the more specific callback.  E.g
--    GE: #preRender -- the #preRender General Event Callback
--
--   This is then followed by a timeStamp() for timing checks.
--
--   For General Event callbacks, where applicable, the
--   callbacks.notificationParams() result(s) are printed out.  If there is
--   more than one result, each result is numbered for easier look-up in
--   MaxScript / SDK documentation.
--
-- ---------------------------=( Revision History )=---------------------------
-- v0.1
--  - initial release
--
-- -----------------------------=( Known Issues )=-----------------------------
-- None
--
-- --------------------------------=( SCRIPT )=--------------------------------
		-- these two functions can't be inside a struct; the callbacks throw horrid errors
		global cbLogger, cbLogger_timeCB, cbLogger_redrawViewsCB
		fn cbLogger_timeCB = (
				format "TM: % @ %\n" #time (timeStamp()) to:cbLogger.logInto
		)
		fn cbLogger_redrawViewsCB = (
			format "RV: % @ %\n" #redrawViews (timeStamp()) to:cbLogger.logInto
		)

		Struct cbLogger (
			logInto = Listener,

			fn cb type = (
				format "GE: % @ %\n" type (timeStamp()) to:cbLogger.logInto
				local params = callbacks.notificationParam()
				if (params != undefined) do (
					if (classOf params != array) then ( format "\t%\n" params to:cbLogger.logInto )
					else (
						for i = 1 to params.count do ( format "\t[%] %\n" i params[i] to:cbLogger.logInto )
					)
				)
			),

			fn add type = (
				try ( callbacks.addScript type ("cbLogger.cb #" + type as string) id:#cbLogger to:cbLogger.logInto )
				catch (
					-- callback type not supported
				)
			),
			
			nodeEventCBvar,
			fn nodeEventCB type objs = (
				format "NE: % @ %\n" type (timeStamp()) to:cbLogger.logInto
				for i = 1 to objs.count do (
					format "\t[%] %\n" i objs[i] to:cbLogger.logInto
				)
			),
			
			fn changeHandlerCB type = (
				format "CH: % @ %\n" type (timeStamp()) to:cbLogger.logInto
			),

			fn start types: chObj: into:Listener = (
				logInto = into
				if (types == unsupplied) then ( types = #(#all) )
				local all = ((findItem types #all) != 0)
			
				if (all OR ((findItem types #animation) != 0)) do (
					cbLogger.add #animateOn
					cbLogger.add #animateOff
					cbLogger.add #animationRangeChange
				)
				if (all OR ((findItem types #renderer) != 0)) do (
					cbLogger.add #tabbedDialogCreated
					cbLogger.add #tabbedDialogDeleted
					cbLogger.add #renderParamsChanged
					cbLogger.add #preRendererChange
					cbLogger.add #postRendererChange
				)
				if (all OR ((findItem types #imageViewer) != 0)) do (
					cbLogger.add #preImageViewerDisplay
					cblogger.add #postImageViewerDisplay
				)
				if (all OR ((findItem types #rendering) != 0)) do (
					cbLogger.add #preRender
					cbLogger.add #preRenderEval
					cbLogger.add #preRenderFrame
					cbLogger.add #beginRenderingActualFrame
					cbLogger.add #beginRenderingReflectRefractMap
					cbLogger.add #beginRenderingTonemappingImage
					cbLogger.add #postRenderFrame
					cbLogger.add #postRender
				)
				if (all OR ((findItem types #fileLink) != 0)) do (
					cbLogger.add #fileLinkPreAttach
					cbLogger.add #fileLinkPostAttach
					cbLogger.add #fileLinkPreBind
					cbLogger.add #fileLinkPostBind
					cbLogger.add #fileLinkPreDetatch
					cbLogger.add #fileLinkPostDetatch
					cbLogger.add #fileLinkPreReload
					cbLogger.add #fileLinkPostReload
					cbLogger.add #filelinkPostReloadPrePrune
				)
				if (all OR ((findItem types #file) != 0)) do (
					cbLogger.add #filePreOpen
					cbLogger.add #filePostOpen
					cbLogger.add #filePreOpenProcess
					cbLogger.add #filePostOpenProcess
					cbLogger.add #filePreSave
					cbLogger.add #filePostSave
					cbLogger.add #filePreSaveProcess
					cbLogger.add #filePostSaveProcess
					cbLogger.add #filePreSaveOld
					cbLogger.add #filePostSaveOld
					cbLogger.add #filePreMerge
					cbLogger.add #filePostMerge
					cbLogger.add #filePostMergeProcess
					cbLogger.add #preImport
					cbLogger.add #postImport
					cbLogger.add #importFailed
					cbLogger.add #preExport
					cbLogger.add #postExport
					cbLogger.add #exportFailed
				)
				if (all OR ((findItem types #xrefs) != 0)) do (
					cbLogger.add #objectXrefPreMerge
					cbLogger.add #objectXrefPostMerge
					cbLogger.add #sceneXrefPreMerge
					cbLogger.add #sceneXrefPostMerge
				)
				if (all OR ((findItem types #layer) != 0)) do (
					cbLogger.add #layerCreated
					cbLogger.add #layerDeleted
					cbLogger.add #nodeLayerChanged
				)
				if (all OR ((findItem types #manipulate) != 0)) do (
					cbLogger.add #manipulateModeOn
					cbLogger.add #manipulateModeOff
				)
				if (all OR ((findItem types #modpanel) != 0)) do (
					cbLogger.add #modPanelObjPreChange
					cbLogger.add #modPanelObjPostChange
					cbLogger.add #modPanelSelChanged
					cbLogger.add #ModPanelSubObjectLevelChanged
				)
				if (all OR ((findItem types #modifier) != 0)) do (
					cbLogger.add #preModifierAdded
					cbLogger.add #postModifierAdded
					cbLogger.add #preModifierDeleted
					cbLogger.add #postModifierDeleted
				)
				if (all OR ((findItem types #mtllib) != 0)) do (
					cbLogger.add #mtlLibPreOpen
					cbLogger.add #mtlLibPostOpen
					cbLogger.add #mtlLibPreSave
					cbLogger.add #mtlLibPostSave
					cbLogger.add #mtlLibPreMerge
					cbLogger.add #mtlLibPostMerge
				)
				if (all OR ((findItem types #material) != 0)) do (
					cbLogger.add #mtlRefAdded
					cbLogger.add #mtlRefDeleted
				)
				if (all OR ((findItem types #node) != 0)) do (
					cbLogger.add #nodeCreated
					cbLogger.add #nodeCloned
					cbLogger.add #sceneNodeAdded
					cbLogger.add #nodeFreeze
					cbLogger.add #nodeUnfreeze
					cbLogger.add #nodeHide
					cbLogger.add #nodeUnhide
					cbLogger.add #nodeLinked
					cbLogger.add #nodeUnlinked
					cbLogger.add #nodePreDelete
					cbLogger.add #nodePostDelete
					cbLogger.add #nodePreMaterial
					cbLogger.add #nodePostMaterial
					cbLogger.add #nodeRenamed
					cbLogger.add #nodeNameSet
					cbLogger.add #preNodeBonePropChanged
					cbLogger.add #postNodeBonePropChanged
					cbLogger.add #preNodeGeneralPropChanged
					cbLogger.add #postNodeGeneralPropChanged
					cbLogger.add #preNodeGiPropChanged
					cbLogger.add #postNodeGiPropChanged
					cbLogger.add #preNodeMentalrayPropChanged
					cbLogger.add #postNodeMentalrayPropChanged
					cbLogger.add #preNodesCloned
					cbLogger.add #postNodesCloned
					cbLogger.add #preNodeUserPropChanged
					cbLogger.add #postNodeUserPropChanged
					cbLogger.add #preMirrorNodes
					cbLogger.add #postMirrorNodes
					cbLogger.add #selectedNodesPreDelete
					cbLogger.add #selectedNodesPostDelete
					cbLogger.add #selectionSetChanged
				)
				if (all OR ((findItem types #system) != 0)) do (
					cbLogger.add #postSystemStartup
					cbLogger.add #systemPreNew
					cbLogger.add #systemPostNew
					cbLogger.add #systemPreReset
					cbLogger.add #systemPostReset
					cbLogger.add #systemPreDirChange
					cbLogger.add #systemPostDirChange
					cbLogger.add #preSystemShutdown
					cbLogger.add #postSystemShutdown
					cbLogger.add #pluginLoaded
				)
				if (all OR ((findItem types #radiosity) != 0)) do (
					cbLogger.add #radiosityPluginChanged
					cbLogger.add #radiosityProcessDone
					cbLogger.add #radiosityProcessReset
					cbLogger.add #radiosityProcessStart
					cbLogger.add #radiosityProcessStopped
				)
				if (all OR ((findItem types #undo) != 0)) do (
					cbLogger.add #sceneUndo
					cbLogger.add #sceneRedo
				)
				if (all OR ((findItem types #schematicView) != 0)) do (
					cbLogger.add #svSelectionSetChanged
					cbLogger.add #svDoubleClickGraphNode
					cbLogger.add #svPreLayoutChange
					cbLogger.add #svPostLayoutChange
				)
				if (all OR ((findItem types #assetBrowser) != 0)) do (
					cbLogger.add #assetBrowserPreNavigate
				)
				if (all OR ((findItem types #namedSelSet) != 0)) do (
					cbLogger.add #NamedSelSetCreated
					cbLogger.add #NamedSelSetDeleted
					cbLogger.add #NamedSelSetRenamed
				)
				if (all OR ((findItem types #direct3d) != 0)) do (
					cbLogger.add #D3DPreDeviceReset
					cbLogger.add #D3DPostDeviceReset
				)
				if (all OR ((findItem types #other) != 0)) do (
					cbLogger.add #bitmapChanged
					cbLogger.add #byCategoryDisplayFilterChanged
					cbLogger.add #colorChanged
					cbLogger.add #customDisplayFilterChanged
					cbLogger.add #lightingUnitDisplaySystemChange
					cbLogger.add #spacemodeChange
					cbLogger.add #timeunitsChange
					cbLogger.add #unitsChange
					cbLogger.add #viewportChange
					cbLogger.add #mainWindowEnabled
					cbLogger.add #heightMenuChanged
					cbLogger.add #preProgress
					cbLogger.add #postProgress
				)
				
				if (all OR ((findItem types #nodeEvent) != 0)) do (
					if (nodeEventCallback != undefined) then (
						cbLogger.nodeEventCBvar = undefined
						gc light:true
						cbLogger.nodeEventCBvar = nodeEventCallback all:cbLogger.nodeEventCB
					)
				)
				
				if (all OR ((findItem types #changeHandlers) != 0)) do (
					if (chObj == unsupplied) then ( chObj = getCurrentSelection() )
					when topology chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #topology )
					when geometry chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #geometry  )
					when name chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #name  )
					when transform chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #transform  )
					when select chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #select  )
					when parameters chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #parameters  )
					when subAnimStructure chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #subAnimStructure  )
					when controller chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #controller  )
					when children chObj changes id:#cbLogger do ( cbLogger.changeHandlerCB #children  )
					when chObj deleted id:#cbLogger do ( cbLogger.changeHandlerCB #deleted  )
				)
				
				if (all OR ((findItem types #redrawViews) != 0)) do (
					registerRedrawViewsCallback cbLogger_redrawViewsCB
				)
				
				if (all OR ((findItem types #time) != 0)) do (
					registerTimeCallback cbLogger_timeCB
				)

				OK
			), -- start()
			fn stop = (
				callbacks.removeScripts id:#cbLogger
				deleteAllChangeHandlers id:#cbLogger
				unregisterRedrawViewsCallback cbLogger_redrawViewsCB
				unregisterTimeCallback cbLogger_timeCB
				cbLogger.nodeEventCBvar = undefined
				gc light:true
				OK
			)
		) -- cbLogger struct
		cbLogger = cbLogger()

		rollout roll_cbLogger "cbLogger - v0.1" width:208 height:456 (
			local events = #(
				#nodeEvent,
				#animation,
				#renderer,
				#imageViewer,
				#rendering,
				#fileLink,
				#file,
				#xrefs,
				#layer,
				#manipulate,
				#modpanel,
				#modifier,
				#mtllib,
				#material,
				#node,
				#system,
				#radiosity,
				#undo,
				#schematicView,
				#assetBrowser,
				#namedSelSet,
				#direct3d,
				#changeHandlers,
				#redrawViews,
				#time
			)
			checkbutton ckb_log "log events" pos:[8,8] width:192 height:16 highlightColor:(color 192 0 0)
			radioButtons rdo_logto "" pos:[8,32] width:143 height:16 labels:#("listener", "window") columns:2
			multiListBox mbx_events "" pos:[8,56] width:192 height:26 items:events selection:#{1..events.count}
			groupBox grp_ch "expression for changeHandlers" pos:[8,408] width:192 height:40
			editText edt_ch "" text:"getCurrentSelection()" pos:[16,424] width:176 height:16

			fn getSelEvents = (
				result = #()
				for i = 1 to events.count do (
					if (mbx_events.selection[i]) then ( append result events[i] )
				)
				result
			)
			on ckb_log changed state do (
				if (state) then (
					cbLogger.stop()
					local chObjError = false
					local chObj
					try ( chObj = execute edt_ch.text )
					catch ( chObjError = true )
					if (classOf chObj != Array) then (
						if (not (isKindOf chObj MaxWrapper)) then ( chObjError = true )
					)
					else (
						for o in chObj do (
							if (not (isKindOf o MaxWrapper)) then ( chObjError = true )
						)
					)
					if (chObjError) do (
						format "cbLogger: changeHandler expression invalid!  Must result in MaxWrapper or array of MaxWrappers!\n"
						chObj = #()
					)
					
					local logTarget = case rdo_logto.state of (
						1: Listener
						2: (newScript())
					)
					cbLogger.start types:(getSelEvents()) chObj:chObj into:logTarget
				)
				else ( cbLogger.stop() )
			)
			on rdo_logto changed state do ( ckb_log.changed ckb_log.state )
			on mbx_events selectionEnd do ( ckb_log.changed ckb_log.state )
			on edt_ch entered str do ( ckb_log.changed ckb_log.state )
			
			on roll_cbLogger close do (
				cbLogger.stop()
			)
		) -- roll_cbLogger


macroScript cbLogger category:"SplutterFish" buttonText:"cbLogger" tooltip:"cbLogger - Log callback events" (
	on isVisible do ( cbLogger != undefined )
	on isEnabled do ( cbLogger != undefined )
	on execute do (
		if (cbLogger == undefined) do ( messageBox "Please re-install cbLogger" title:"cbLogger definition missing" )
		createDialog roll_cbLogger style:#(#style_titlebar,#style_sysmenu,#style_minimizebox)
	)
-- --------------------------------=( /SCRIPT )=-------------------------------
) -- macroscript
